/*
   PMS_APP.CPP
   PageMgr Sample Application functions.

   Copyright (c) Bachmann Software and Services, 1998
   Author: Greg Winton
*/

#include <Pilot.h>
#include <SysEvtMgr.h>
#include "pms.h"
#include "pms_res.h"


// Private constants.
const unsigned long  appFileCreator = PMS_FILE_CREATOR;
const unsigned short appFileVersion = PMS_FILE_VERSION;
const unsigned short appPrefID      = PMS_PREF_ID;
const unsigned short appPrefVersion = PMS_PREF_VERSION;
const unsigned long  appOSVersion   = PALMOS_VERSION;
const unsigned long  osVersion20    = PALMOS_VERSION_20;



// Private structures.

/*
   AppPreferences:
   Application preferences.
   Members: 
*/
struct AppPreferences
{
   Byte byReplaceMe;
};

/*
   AppInformation:
   Application information.
   Members:
*/
struct AppInformation
{
   Byte  byReplaceMe;
};

// Private functions.
static Err     AppStart ();
static void    AppStop ();
static void    AppEventLoop (void);
static Boolean AppEventHandler (EventPtr);
static void    AppLoadForm (Word);
static void    AppReadPreferences ();
static void    AppWritePreferences ();
static Err     AppCheckVersion (DWord, Word);

// Private data.
static AppPreferences   s_appPref;
static AppInformation   s_appInfo;

/*
   PilotMain ()
   Main entry point into pilot application.
   Parms:   wCmd  - launch code.
            pInfo - struct associated w/ launch code.
            fuLaunch - extra info about launch.
*/
DWord 
PilotMain (Word wCmd, Ptr /*pInfo*/, Word fuLaunch)
{
   Err   err;

   // Check version compatibility.
   err = AppCheckVersion (appOSVersion, fuLaunch);
   if (err)
   {
      // Incompatible - exit.
      return err;
   }
   
   switch (wCmd)
   {
      case sysAppLaunchCmdNormalLaunch:
      {
         // Initialize application.
         err = AppStart ();
         if (!err)
         {
            // Show the main form.
            FrmGotoForm (MainForm);
            AppEventLoop ();
            AppStop ();
         }
         break;
      }
      default:
      {
         err = 0;
         break;
      }
   }
   return err;
}

/*
   AppStart ():
   Initialize the application.
   Parms:   none
   Return:  0 if success, or error code.
*/
Err
AppStart ()
{

   // Read preferences for the app.
   AppReadPreferences ();


   return 0;
}



/*
   AppStop ():
   Save the current state of the application.
   Parms:   none
   Return:  none
*/
void
AppStop ()
{
   AppWritePreferences ();
}
   
/*
   AppEventLoop (void)
   Process events coming to application.
   Parms:   none
   Return:  none
   Note:    Application is in this function for majority
            of its execution.
*/
void
AppEventLoop (void)
{
   EventType   event;
   
   do
   {
      EvtGetEvent (&event, evtWaitForever);
      
      // Ask system to handle event.
      if (false == SysHandleEvent (&event))
      {
         // System did not handle event.
         
         Word        error;
         // Ask Menu to handle event.
         if (false == MenuHandleEvent (0, &event, &error))
         {
            // Menu did not handle event.
            // Ask App (that is, this) to handle event.
            if (false == AppEventHandler (&event))
            {
               // App did not handle event.
               // Send event to appropriate form.
               FrmDispatchEvent (&event);
            }  // end if (false == AppEventHandler (&event))
         }     // end if (false == MenuHandleEvent (0, &event, &error))
      }        // end if (false == SysHandleEvent (&event))
   }
   while (event.eType != appStopEvent);
}

/*
   AppEventHandler (EventPtr)
   Handle application-level events.
   Parms:   pEvent   - event to handle.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
AppEventHandler (EventPtr pEvent)
{
   Boolean bHandled = false;
   
   switch (pEvent->eType)
   {
      case frmLoadEvent:
      {
         // Load the form resource.
         Word  wFormID = pEvent->data.frmLoad.formID;
         
         AppLoadForm (wFormID);
         bHandled = true;
         break;
      }

      default:
      {
         break;
      }
   }
   return bHandled;
}

/*
   AppLoadForm (Word)
   Load a form.
   Parms:   wFormID  - form to be loaded.
   Return:  none
*/
void
AppLoadForm (Word wFormID)
{
   FormPtr pForm = FrmInitForm (wFormID);
   
   FrmSetActiveForm (pForm);
   
   // Set the event handler for the form.
   //    The handler of the currently active form is called by
   //    FrmHandleEvent each time it receives an event.
   switch (wFormID)
   {
      case MainForm:
      {
         FrmSetEventHandler (pForm, MainFormEventHandler);
         break;
      }

      default:
      {
         break;
      }
   }
}

void
AppReadPreferences ()
{
   Word  cbPref = sizeof s_appPref;
   Err   err = PrefGetAppPreferences (appFileCreator, 
                                       appPrefID,
                                       &s_appPref,
                                       &cbPref,
                                       true);
   if (err == noPreferenceFound)
   {
      // Clear the app preferences.
      MemSet (&s_appPref, sizeof (s_appPref), '\0');
      
      // App started okay, just no preferences.
      err = 0;
   }
}

void
AppWritePreferences ()
{
   // Read preferences.
   PrefSetAppPreferences (appFileCreator,
                          appPrefID,
                          appPrefVersion,
                          &s_appPref,
                          sizeof (s_appPref),
                          true);
}

/*
   AppCheckVersion ():
   Check hardware compatibility.
   Parms:   dwReqVersion   - required ROM version.
            fuLaunch       - launch flags.
   Return:  0 if ROM is compatible or error code.
*/
Err
AppCheckVersion (DWord dwReqVersion, Word fuLaunch)
{
   // Assume the best.
   Err err = 0;
   
   // Get the ROM version.
   DWord dwRomVersion;
   
   FtrGet (sysFtrCreator, sysFtrNumROMVersion, &dwRomVersion);
   
   // Check if its compatible.
   if (dwRomVersion < dwReqVersion)
   {
      // Version is too low, see what launch flags are.
      Word  wLaunchCheck = sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp;
      
      if (wLaunchCheck & fuLaunch == wLaunchCheck)
      {
         // Tell user the problem.
			FrmAlert (RomIncompatibleAlert);
		
			// Pilot 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (dwRomVersion < osVersion20)
			{
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
			}
		}
      err = sysErrRomIncompatible;
   }
   return err;
}
   
