/*
   PMS_MAIN.CPP
   Main form handling functions.
   Copyright (c) Bachmann Software and Services, 1998
   Author: Glenn Bachmann
*/

#include <Pilot.h>
#include <SysEvtMgr.h>
#include <PrintMgr.h>
#include "pms.h"
#include "pms_res.h"

static Char printerStatic [50];
static Int printtype = 0;

typedef enum {
   printMgrLoaded,
   printMgrOpened,
   printMgrUnloaded
} PrintMgrStateEnum;

static void    MainFormInit (FormPtr formP);
static Boolean MainFormButtonHandler (FormPtr formP, EventPtr eventP);
static Boolean MainFormMenuHandler (FormPtr formP, EventPtr eventP);
static UInt    MainFormGetPrintMgrRefNum (FormPtr formP);
static void    MainFormSetPrintMgrRefNum (FormPtr formP, UInt refNum);
static void    MainFormEnableControls (FormPtr formP, PrintMgrStateEnum printMgrState);
static void    MainFormEnableControl (FormPtr formP, Word objectID, Boolean enabled);

// Print Manager function wrappers.
static Err     SampleLoad (FormPtr formP);
static Err     SampleUnload (FormPtr formP);
static Err     SampleVersion (FormPtr formP);
static Err     SampleOpen (FormPtr formP);
static Err     SampleClose (FormPtr formP);
static Err     SamplePrint (FormPtr formP);
static Err     SampleLines (FormPtr formP);
static Err     SampleRectangles (FormPtr formP);
static Err     SampleFancyText (FormPtr formP);
static Err     SampleFontText (FormPtr formP);
static Err     SampleSettings (FormPtr formP);
static Err     SampleNewPage (FormPtr formP);
static Err     SampleVersion (FormPtr formP);
static Err     SampleSimpleTextAndGraphics (FormPtr formP);
static Err     SampleOneil(FormPtr formP);



/*
   MainFormEventHandler:
   Parms:   pEvent   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormEventHandler (EventPtr eventP)
{
   Boolean  handled = false;
   
   switch (eventP->eType)
   {
      case menuEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormMenuHandler (formP, eventP);
         break;
      }
      
      case ctlSelectEvent:
      {
      // A control button was pressed.
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormButtonHandler (formP, eventP);
         break;
      }
      
      case frmOpenEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         MainFormInit (formP);
         FrmDrawForm (formP);
         handled = true;
         break;
      }
      
      default:
      {
         break;
      }
   }
   return handled;
}

/*
   MainFormInit:
   Initialize the main form.
   Parms:   formP - pointer to main form.
   Return:  none
*/
void
MainFormInit (FormPtr formP)
{
   // Enable the appropriate controls.
   MainFormEnableControls (formP, printMgrUnloaded);
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            command  - command to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormMenuHandler (FormPtr /*formP*/, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.menu.itemID)
   {
      default:
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            eventP   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormButtonHandler (FormPtr formP, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.ctlEnter.controlID)
   {
      case MainLoadButton:   // User selected Load button.
      {
         SampleLoad (formP);
         handled = true;
         break;
      }
   
      case MainUnloadButton:   // User selected Unload button.
      {
         SampleUnload (formP);
         handled = true;
         break;
      }

      case MainOpenButton:
      {
         SampleOpen (formP);
         handled = true;
         break;
      }
      case MainPrintButton:   // User selected Print button.
      {
         SamplePrint (formP);
         handled = true;
         break;
      }


      case MainCloseButton:   // User selected Close button.
      {
         SampleClose (formP);
         handled = true;
         break;
      }
      
      case MainVersionButton:   // User selected Version button.
      {
         SampleVersion (formP);
         handled = true;
         break;
      }
   }
   return handled;
}
         

/*
   SampleLoad:
   Sample code for loading the Print Manager shared library.
   Parms:   formP                   Parent form.
   Return:  0                       Success.
            sysErrLibNotFound       Print Manager library not installed.
            sysErrNoFreeRAM         System error loading shared library.
            sysErrNoFreeLibSlots    System error loading shared library.
            prnErrAlreadyLoad       Library has already been loaded.
*/
Err
SampleLoad (FormPtr formP)
{
   // Declare text buffer for messages.
   Char  msg [80];
   UInt  msgSize = sizeof (msg);
   
   // Print Manager may only be loaded once, so we do not try
   // to find it first.  Already loaded is an error in this case.
   UInt refNum = 0;
   Err err = SysLibLoad (printMgrTypeID, printMgrCreatorID, &refNum);
   
   // Check result of loading library.
   if (!err)
   {
   // Print Manager loaded successfully.
   
      // Store ref num.
      MainFormSetPrintMgrRefNum (formP, refNum);
      
      // Enable the appropriate controls.
      MainFormEnableControls (formP, printMgrLoaded);
   }
   else
   {
   // Error loading print manager.

      // Notify user of error.
      StrPrintF (msg, "Error %d loading Print Manager.", err);
      FrmCustomAlert (ErrorAlert, msg, NULL, NULL);
   }
   
   // Return result.
   return err;
}

/*
   SampleUnload:
   Unload the print manager library.
   Parms:   formP       Parent form.
   Return:  0           Success
*/
Err
SampleUnload (FormPtr formP)
{
   // Initialize result.
   Err err = 0;
   
   // Retrieve ref num for PrintMgr library.
   UInt refNum = MainFormGetPrintMgrRefNum (formP);
   
   // Make sure refNum is not 0 (we should not be here if it is).
   ErrFatalDisplayIf ((refNum == 0), "Invalid state!");

   // Unload PrintMgr library.
   err = SysLibRemove (refNum);

   // Update controls (we are back to unloaded state).
   MainFormEnableControls (formP, printMgrUnloaded);
   
   // Return result.
   return err;
}

/*
   SampleOpen:
   Sample code for opening the Print Manager shared library.
   Parms:   formP                   Parent form.
   Return:  0                       Success.
            prnErrAlreadyOpen       Library has already been loaded.
*/
Err
SampleOpen (FormPtr formP)
{
   // Declare text buffer for messages.
   Char  buf[80];
   Err err;
   UInt refNum = 0;


   // Retrieve ref num for PrintMgr shared library.
   refNum = MainFormGetPrintMgrRefNum (formP);
   
   // Make sure refNum is not 0 (we should not be here if it is).
   ErrFatalDisplayIf ((refNum == 0), "Invalid state!");

   // Open print manager.
   err = PrnOpenByConfig(refNum);
   if (!err)
   {
      // Print Manager opened successfully.
      PrnGetConfigValue(refNum,"DRIVERNAME",printerStatic,sizeof(printerStatic));
      // Update controls.
      MainFormEnableControls (formP, printMgrOpened);
   }
   else
   {
   // Error opening print manager.

      // Notify user of error.
      MemSet(buf,sizeof(buf),0x00);
      StrPrintF (buf, "Error [%d] Opening Print Manager.", err);
      FrmCustomAlert (SuccessAlert, buf, "", "");
   }
   
   // Return result.
   return err;
}

/*
   SampleClose:
   Close the print manager library.
   Parms:   formP       Parent form.
   Return:  0           Success
*/
Err
SampleClose (FormPtr formP)
{
   // Initialize result.
   Err err = 0;
   
   // Retrieve ref num for PrintMgr library.
   UInt refNum = MainFormGetPrintMgrRefNum (formP);
   
   // Check if ref num is valid.
   if (refNum)
   {
   // Ref num is valid.
   
      // Close Print Manager.
      PrnClose (refNum);

      // Update controls (we are back to loaded state).
      MainFormEnableControls (formP, printMgrLoaded);
      
      // Set result to success.
      err = 0;
   }
   else
   {
   // Ref num is not valid.
   
      // Set result to state error.
      err = prnErrState;
   }
   
   // Return result.
   return err;
}

/*
   SamplePrint1:
   Print using the print manager library.
   Parms:   formP       Parent form.
   Return:  0           Success
*/
Err
SamplePrint (FormPtr formP)
{
   if (0 == StrCaselessCompare (printerStatic, "PCL"))
   {
      // Set settings.
      SampleSettings (formP);
      
      // Print Lines sample.
      SampleLines (formP);
      
      // Print Black Rectangles sample.
      SampleRectangles (formP);
      
      // Print Text Drawing sample.
      SampleFancyText (formP);
      
      // New page sample.
      SampleNewPage (formP);
   }
   else if (0 == StrCaselessCompare (printerStatic, "Canon"))
   {
      // Print formatted text sample
      SampleFontText (formP);

      // New page
      SampleNewPage (formP);
   }
   else if (0 == StrCaselessCompare (printerStatic, "PocketJet"))
   {
   	SampleSimpleTextAndGraphics (formP);
      SampleNewPage(formP);
   }
   else if (0 == StrCaselessCompare (printerStatic, "Oneil"))
   {
      SampleOneil(formP);
      SampleNewPage(formP);
   }
   else
   {
      // Print simple text sample
      // Font selection will be ignored.
      SampleFontText (formP);

      // New page
      SampleNewPage (formP);
   }
   // Return success.
   return 0;   
}

/*
   SampleLines:
   Draw a collection of gray lines.
   Parms:   formP       Parent form.
   Return:  0           Success
   Note:    This sample function draws a square in which the top
            and left lines are gray, while the right and botom
            lines are black.
*/
Err
SampleLines (FormPtr formP)
{
   // Initialize error.
   Err err = 0;
   
   // Retrieve PrintMgr reference number.
   UInt refNum = MainFormGetPrintMgrRefNum (formP);
   
   // Make sure refNum is not 0 (we should not be here if it is).
   ErrFatalDisplayIf ((refNum == 0), "Invalid state!");

   // Line samples start at (50, 900).
   // NB: At 300 DPI (most printers), 900 pixels is 3".
   const Int startX = 50;
   const Int startY = 50;
   const Int extentX = 1400;
   const Int extentY = 700;
   
   Int x1, y1, x2, y2;
   
   // Draw top line - horizontal gray:
   x1 = startX;
   x2 = startX + extentX;
   y1 = y2 = startY;    // Same value for y1, y2 - top of rectangle.
   PrnDrawGrayLine (refNum, x1, y1, x2, y2);
   
   // Draw left line - vertical gray:
   x1 = x2 = startX;    // Same value for x1, x2
   y1 = startY;
   y2 = startY + extentY;
   PrnDrawGrayLine (refNum, x1, y1, x2, y2);
   
   // Draw first diagonal - gray
   x1 = startX;
   y1 = startY;
   x2 = startX + extentX;
   y2 = startY + extentY;
   PrnDrawGrayLine (refNum, x1, y1, x2, y2);

   // Draw bottom line - horizontal black:
   x1 = startX;
   x2 = startX + extentX;
   y1 = y2 = startY + extentY;   // Same value for y1, y2 - bottom of rectangle.
   PrnDrawLine (refNum, x1, y1, x2, y2);
   
   // Draw right line - vertical black
   // Note: This line draws from bottom up
   x1 = x2 = startX + extentX;   // Same value for x1, x2 - right of rectangle
   y1 = startY + extentY;
   y2 = startY;
   PrnDrawLine (refNum, x1, y1, x2, y2);
   
   // Draw second diagonal - black:
   x1 = startX + extentX;
   y1 = startY;
   x2 = startX;
   y2 = startY + extentY;
   PrnDrawLine (refNum, x1, y1, x2, y2);

   // Return result.
   return err;
}

/*
   SampleRectangles:
   Draw a collection of gray rectangles.
   Parms:   formP       Parent form.
   Return:  0           Success
*/
Err
SampleRectangles (FormPtr formP)
{
   // Initialize error.
   Err err = 0;
   
   // Retrieve PrintMgr reference number.
   UInt refNum = MainFormGetPrintMgrRefNum (formP);
   
   // Make sure refNum is not 0 (we should not be here if it is).
   ErrFatalDisplayIf ((refNum == 0), "Invalid state!");

   // Origin constants - we want to start rectangles to left of lines
   //                    (see SampleLines, above).
   const Int originX = 1500;
   
   // Extent constants - each rectangle is 300 by 300.
   const Int extentX = 300;
   const Int extentY = 300;
   
   // Margin constants - we'll put a 50 pixel border around each.
   const Int marginX = 50;
   const Int marginY = 50;

   RectangleType r;
   
// Draw alternating solid and frame rectangles.
   
   // Paint first rectangle (solid).
   r.topLeft.x = originX;                 // Start at left margin.
   r.topLeft.y = marginY;                 // Start at top margin.
   r.extent.x = extentX;                  // 300 pixels wide.
   r.extent.y = extentY;                  // 300 pixels high.
   PrnDrawRectangle (refNum, &r, 0);      // Corner Diameters not supported.
   
   // Paint second rectangle (frame).
   r.topLeft.x += (extentX + marginX);    // Move right of first rectangle.
   r.topLeft.y = marginY;                 // Still at top margin.
   r.extent.x = extentX;                  // 300 pixels wide.
   r.extent.y = extentY;                  // 300 pixels high.
   PrnDrawRectangleFrame (refNum, &r);    // Corner Diameters not supported.

// Draw alternating gray solid and frame rectangles.

   // Paint first rectangle (solid).
   r.topLeft.x = originX;                       // Start at left margin.
   r.topLeft.y = marginY + (extentY + marginY); // Start one down from prev
   r.extent.x = extentX;                        // 300 pixels wide.
   r.extent.y = extentY;                        // 300 pixels high.
   PrnDrawGrayRectangle (refNum, &r, 0);        // Corner Diameters not supported.
   
   // Paint second rectangle (frame).
   r.topLeft.x += (extentX + marginX);          // Move right of first rectangle.
   r.topLeft.y = marginY + (extentY + marginY); // Start one down from prev
   r.extent.x = extentX;                        // 300 pixels wide.
   r.extent.y = extentY;                        // 300 pixels high.
   PrnDrawGrayRectangleFrame (refNum, &r);      // Corner Diameters not supported.

   // Return results.
   return err;
}

/*
   SampleFancyText:
   Exercise text handling.
   Parms:   formP       Parent form.
   Return:  0           Success
*/
Err
SampleFancyText (FormPtr formP)
{
// Retrieve PrintMgr reference number.

   // Initialize error.
   Err err = 0;
   
   // Retrieve PrintMgr reference number.
   UInt refNum = MainFormGetPrintMgrRefNum (formP);

   // Make sure refNum is not 0 (we should not be here if it is).
   ErrFatalDisplayIf ((refNum == 0), "Invalid state!");

// Draw caption.

   // Define starting positions:
   Int startX = 50;
   Int startY = 900;
   Int extentX = 1800;
   Int extentY = 300;
   Int charHeight = 0;
   
   // Set font to TimesRoman 20 pt.
   PrnFontType font;
   StrCopy (font.typeface, "Times New Roman"); 
   font.height = 14;
   font.bold = true;
   font.italics = false;
   err = PrnSetFont (refNum, &font);

   // Print line.
   CharPtr string = "Text and Font Handling:";
   err = PrnDrawChars (refNum, string, StrLen (string), startX, startY);
   
   // Get the font height (we'll need this later).
   err = PrnGetCharHeight (refNum, &charHeight);
   
// Now do some formatted text printing.

   // Define text rectangle.
   RectangleType r;
   r.topLeft.x = startX;
   r.topLeft.y = startY + (2*charHeight);  // Start below caption.
   r.extent.x = extentX;
   r.extent.y = extentY - (2*charHeight);

   // Print a line in Courier 10 pt.
   StrCopy (font.typeface, "Courier"); 
   font.height = 10;

   font.bold = false;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = "Courier, 10 point";
   err = PrnDrawText (refNum, string, StrLen (string), &r, prnDrawTextLeft | prnDrawTextTop);

   font.bold = true;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = "Bold";
   err = PrnDrawText (refNum, string, StrLen (string), &r, prnDrawTextCenter | prnDrawTextTop);
   
   font.bold = false;
   font.italics = true;
   err = PrnSetFont (refNum, &font);
   string = "Italic";
   err = PrnDrawText (refNum, string, StrLen (string), &r, prnDrawTextRight | prnDrawTextTop);
   
   // Print a line in Courier 10 pt.
   StrCopy (font.typeface, "Courier"); 
   font.height = 12;

   font.bold = false;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = "Courier, 12 point";
   err = PrnDrawText (refNum, string, StrLen (string), &r, prnDrawTextLeft | prnDrawTextVCenter);

   font.bold = true;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = "Bold";
   err = PrnDrawText (refNum, string, StrLen (string), &r, prnDrawTextCenter | prnDrawTextVCenter);
   
   font.bold = false;
   font.italics = true;
   err = PrnSetFont (refNum, &font);
   string = "Italic";
   err = PrnDrawText (refNum, string, StrLen (string), &r, prnDrawTextRight | prnDrawTextVCenter);
   
   // Print a line in Courier 14 pt.
   StrCopy (font.typeface, "Courier"); 
   font.height = 14;

   font.bold = false;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = "Courier, 12 point";
   err = PrnDrawText (refNum, string, StrLen (string), &r, prnDrawTextLeft | prnDrawTextBottom);

   font.bold = true;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = "Bold";
   err = PrnDrawText (refNum, string, StrLen (string), &r, prnDrawTextCenter | prnDrawTextBottom);
   
   font.bold = false;
   font.italics = true;
   err = PrnSetFont (refNum, &font);
   string = "Italic";
   err = PrnDrawText (refNum, string, StrLen (string), &r, prnDrawTextRight | prnDrawTextBottom);
   
   r.topLeft.x = startX;
   r.topLeft.y = (startY + (2*charHeight)) + extentY;  // Start below caption.
   r.extent.x = extentX;
   r.extent.y = extentY - (2*charHeight);

   // Print a line in times roman 10 pt.
   StrCopy (font.typeface, "Times Roman"); 
   font.height = 12;

   font.bold = false;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = "Times Roman, 10 point";
   err = PrnDrawText (refNum, string, StrLen (string), &r, prnDrawTextLeft | prnDrawTextTop);

   font.bold = true;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = "Bold";
   err = PrnDrawText (refNum, string, StrLen (string), &r, prnDrawTextCenter | prnDrawTextTop);
   
   font.bold = false;
   font.italics = true;
   err = PrnSetFont (refNum, &font);
   string = "Italic";
   err = PrnDrawText (refNum, string, StrLen (string), &r, prnDrawTextRight | prnDrawTextTop);
   
   // Print a line in times roman 12 pt.
   StrCopy (font.typeface, "Times Roman"); 
   font.height = 12;

   font.bold = false;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = "Times Roman, 12 point";
   err = PrnDrawText (refNum, string, StrLen (string), &r, prnDrawTextLeft | prnDrawTextVCenter);

   font.bold = true;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = "Bold";
   err = PrnDrawText (refNum, string, StrLen (string), &r, prnDrawTextCenter | prnDrawTextVCenter);
   
   font.bold = false;
   font.italics = true;
   err = PrnSetFont (refNum, &font);
   string = "Italic";
   err = PrnDrawText (refNum, string, StrLen (string), &r, prnDrawTextRight | prnDrawTextVCenter);
   
   // Print a line in times roman 14 pt.
   StrCopy (font.typeface, "Times Roman"); 
   font.height = 14;

   font.bold = false;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = "Times Roman, 14 point";
   err = PrnDrawText (refNum, string, StrLen (string), &r, prnDrawTextLeft | prnDrawTextBottom);

   font.bold = true;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = "Bold";
   err = PrnDrawText (refNum, string, StrLen (string), &r, prnDrawTextCenter | prnDrawTextBottom);
   
   font.bold = false;
   font.italics = true;
   err = PrnSetFont (refNum, &font);
   string = "Italic";
   err = PrnDrawText (refNum, string, StrLen (string), &r, prnDrawTextRight | prnDrawTextBottom);
   
   // Return result.
   return err;
}
   

/*
   SampleSimpleTextAndGraphics:
   Exercise text and graphics handling.
   Parms:   formP       Parent form.
   Return:  0           Success
*/
Err
SampleSimpleTextAndGraphics (FormPtr formP)
{
// Retrieve PrintMgr reference number.

   // Initialize error.
   Err err = 0;
   RectangleType r;
   Char buf[100];
   BitmapPtr pbmp;
   VoidHand bitmapH;
   Int i;
   Word type;
   // Retrieve PrintMgr reference number.
   UInt refNum = MainFormGetPrintMgrRefNum (formP);

   // Make sure refNum is not 0 (we should not be here if it is).
   ErrFatalDisplayIf ((refNum == 0), "Invalid state!");


   PrnSetOrientation (refNum, prnOrientPortrait);

   // determine what type of output to generate.
   type = FrmCustomAlert(OutputTypeAlert,"","","");

   if (type == 0)  /* graphics */
   {
      for (i=500;i<575;i+=20)
      {
         r.topLeft.x = 25;
         r.topLeft.y = i;
         r.extent.x = 100;
         r.extent.y = 10;
         PrnDrawRectangle(refNum,&r,0);      
         r.topLeft.x = 1000;
         r.topLeft.y = i;
         r.extent.x = 100;
         r.extent.y = 10;
         PrnDrawRectangle(refNum,&r,0);      
      }
   
      for (i=0;i<2000;i+=100)
      {
         r.topLeft.x = 290;
         r.topLeft.y = i;
         r.extent.x = 200;
         r.extent.y = 20;
         PrnDrawRectangle(refNum,&r,0);      
      }
   }
   else  /* text */
   {
      PrnFontType font;
      StrCopy (font.typeface, "Courier New"); 
      font.height = 10;
      font.bold = false;
      font.italics = false;
      ErrFatalDisplayIf(PrnSetFont (refNum, &font), "Failed to SetFont!");
   
      PrnSetOrientation (refNum, prnOrientPortrait);
   
      r.topLeft.x = 10;
      r.topLeft.y = 10;
      r.extent.x = 1600;
      r.extent.y = 55;
      PrnDrawRectangle(refNum,&r,0);
   
      StrPrintF(buf,"What is Bachmann Print Manager?");
      PrnDrawChars(refNum,buf,StrLen(buf),400,40); 
      StrPrintF(buf,"Most computing platforms include printing support right out of the box.");
      PrnDrawChars(refNum,buf,StrLen(buf),10,80); 
      StrPrintF(buf,"But Palm Computing elected not to include a printing subsystem in their operating");
      PrnDrawChars(refNum,buf,StrLen(buf),10,120); 
      StrPrintF(buf,"system. So although Palm OS based devices include both a serial and infrared port");
      PrnDrawChars(refNum,buf,StrLen(buf),10,160); 
      StrPrintF(buf,"for communications, they cannot produce printed output. ");
      PrnDrawChars(refNum,buf,StrLen(buf),10,200); 
      StrPrintF(buf,"We designed Bachmann Print Manager as the \"missing\" printing subsystem that Palm");
      PrnDrawChars(refNum,buf,StrLen(buf),10,280); 
      StrPrintF(buf,"\"forgot\" to put in their product. Like the Palm's infrared library, serial");
      PrnDrawChars(refNum,buf,StrLen(buf),10,320); 
      StrPrintF(buf,"library and network library, Bachmann Print Manager is a standard Palm OS");
      PrnDrawChars(refNum,buf,StrLen(buf),10,360); 
      StrPrintF(buf,"Shared Library, so it is easily and tightly integrated with any Palm Pilot");
      PrnDrawChars(refNum,buf,StrLen(buf),10,400); 
      StrPrintF(buf,"application.  Bachmann Print Manager gives Palm Computing Platform application");
      PrnDrawChars(refNum,buf,StrLen(buf),10,440); 
      StrPrintF(buf,"developers the power to create stunning printed output directly from their Palm");
      PrnDrawChars(refNum,buf,StrLen(buf),10,480); 
      StrPrintF(buf,"Pilot application.");
      PrnDrawChars(refNum,buf,StrLen(buf),10,520);
      StrPrintF(buf,"Now, with Bachmann Print Manager, you can print any information from your Palm");
      PrnDrawChars(refNum,buf,StrLen(buf),10,620); 
      StrPrintF(buf,"Connected Organizer, using either a serial or infrared connection to popular");
      PrnDrawChars(refNum,buf,StrLen(buf),10,660); 
      StrPrintF(buf,"portable, standalone or networked printers.  There is no longer any need to");
      PrnDrawChars(refNum,buf,StrLen(buf),10,700); 
      StrPrintF(buf,"synchronize data back to the desktop in order to create printed output that is");
      PrnDrawChars(refNum,buf,StrLen(buf),10,740); 
      StrPrintF(buf,"of the caliber you would expect from a Windows application.");
      PrnDrawChars(refNum,buf,StrLen(buf),10,780); 
      StrPrintF(buf,"Visit us at Bachmannsoftware.com.");
      PrnDrawChars(refNum,buf,StrLen(buf),40,1000); 

      // draw bitmap
      bitmapH = DmGetResource(bitmapRsc,PentaxBitmap);
      if (bitmapH != 0)
      {
         pbmp = (BitmapPtr) MemHandleLock(bitmapH);
         if (pbmp != NULL)
         {
            PrnDrawBitmap(refNum,pbmp,1100,40);
            MemHandleUnlock(bitmapH);
         }
         DmReleaseResource(bitmapH);
      }
   }
   return 0;  
}





/*
   SampleFontText:
   Exercise text font handling.
   Parms:   formP       Parent form.
   Return:  0           Success
*/
Err
SampleFontText (FormPtr formP)
{
   // Initialize error.
   Err err = 0;

   // Retrieve PrintMgr reference number.
   UInt refNum = MainFormGetPrintMgrRefNum (formP);

   // Make sure refNum is not 0 (we should not be here if it is).
   ErrFatalDisplayIf ((refNum == 0), "Invalid state!");

   // Set font
   PrnFontType font;
   StrCopy (font.typeface, "Courier New"); 
   font.height = 10;
   font.bold = false;
   font.italics = false;
   ErrFatalDisplayIf(PrnSetFont (refNum, &font), "Failed to SetFont!");


   PrnSetOrientation (refNum, prnOrientPortrait);

  

   // Print line.some text
   CharPtr string = "Text and Font Handling for ";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));
   string = printerStatic;
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));
   string = " printers:\n";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));
   string = "Text and Font Handling:";
//   err = PrnDrawChars (refNum, string, StrLen (string), 25, 25);

   // Print a line in Courier 10 pt.
   StrCopy (font.typeface, "Courier New"); 
   font.height = 10;

   font.bold = false;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = "Courier, 10 point";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));

   font.bold = true;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = " Bold ";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));
   
   font.bold = false;
   font.italics = true;
   err = PrnSetFont (refNum, &font);
   string = " Italic ";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));
   
   // Skip a line
   err = PrnWrite (refNum, (BytePtr) "\r\n", 2);

   // Print a line in Courier 10 pt.
   StrCopy (font.typeface, "Courier"); 
   font.height = 12;

   font.bold = false;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = "Courier, 12 point";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));

   font.bold = true;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = " Bold ";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));
   
   font.bold = false;
   font.italics = true;
   err = PrnSetFont (refNum, &font);
   string = " Italic ";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));
   
   // Skip a line
   err = PrnWrite (refNum, (BytePtr) "\r\n", 2);

   // Print a line in Courier 14 pt.
   StrCopy (font.typeface, "Courier"); 
   font.height = 14;

   font.bold = false;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = "Courier, 14 point";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));

   font.bold = true;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = " Bold ";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));
   
   font.bold = false;
   font.italics = true;
   err = PrnSetFont (refNum, &font);
   string = " Italic ";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));
   
   // Skip a line
   err = PrnWrite (refNum, (BytePtr) "\r\n", 2);

   // Print a line in times roman 10 pt.
   StrCopy (font.typeface, "Times Roman"); 
   font.height = 10;

   font.bold = false;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = "Times Roman, 10 point";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));

   font.bold = true;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = " Bold ";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));
   
   font.bold = false;
   font.italics = true;
   err = PrnSetFont (refNum, &font);
   string = " Italic ";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));
   
   // Skip a line
   err = PrnWrite (refNum, (BytePtr) "\r\n", 2);

   // Print a line in times roman 12 pt.
   StrCopy (font.typeface, "Times Roman"); 
   font.height = 12;

   font.bold = false;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = "Times Roman, 12 point";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));

   font.bold = true;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = " Bold ";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));
   
   font.bold = false;
   font.italics = true;
   err = PrnSetFont (refNum, &font);
   string = " Italic ";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));
   
   // Skip a line
   err = PrnWrite (refNum, (BytePtr) "\r\n", 2);

   // Print a line in times roman 14 pt.
   StrCopy (font.typeface, "Times Roman"); 
   font.height = 14;

   font.bold = false;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = "Times Roman, 14 point";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));

   font.bold = true;
   font.italics = false;
   err = PrnSetFont (refNum, &font);
   string = " Bold ";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));
   
   font.bold = false;
   font.italics = true;
   err = PrnSetFont (refNum, &font);
   string = " Italic ";
   err = PrnWrite (refNum, (BytePtr) string, StrLen (string));
   
   // Skip a line
   err = PrnWrite (refNum, (BytePtr) "\r\n", 2);

   // Return result.
   return err;
}


/*
   SampleOneil:
   Exercise Oneil handling.
   Parms:   formP       Parent form.
   Return:  0           Success
*/
Err
SampleOneil (FormPtr formP)
{
   // Initialize error.
   Err err = 0;
   Char buf[100];

   // Retrieve PrintMgr reference number.
   UInt refNum = MainFormGetPrintMgrRefNum (formP);

   // Make sure refNum is not 0 (we should not be here if it is).
   ErrFatalDisplayIf ((refNum == 0), "Invalid state!");


   // Set orientation to landscape.
//   PrnSetOrientation (refNum, prnOrientLandscape);
   PrnSetOrientation (refNum, prnOrientPortrait);

   // Set font
   PrnFontType font;
   StrCopy (font.typeface, "MF226"); 
   PrnSetFont (refNum, &font);

   StrPrintF(buf,"Item #1 yellow     $10.00");
   PrnDrawChars(refNum,buf,StrLen(buf),10,30);
   StrPrintF(buf,"Item #2 blue       $ 3.15");
   PrnDrawChars(refNum,buf,StrLen(buf),40,30);

   StrCopy(font.typeface, "BC39N,HIGH 20, WIDE 2");
   PrnSetFont (refNum, &font);
   StrPrintF(buf,"$13.15");
   PrnDrawChars(refNum,buf,StrLen(buf),100,30);

   StrCopy(font.typeface, "MF107");
   PrnSetFont (refNum, &font);
   StrPrintF(buf,"TOTAL $13.15");
   PrnDrawChars(refNum,buf,StrLen(buf),200,30);

   return err;
}
   
/*
   SampleNewPage:
   Set default settings for the print job - orientation, font, etc.
   Parms:   refNum      Library reference number.
   Return:  0           Success
*/
Err
SampleNewPage (FormPtr formP)
{
   // Initialize error.
   Err err = 0;

   // Retrieve PrintMgr reference number.
   UInt refNum = MainFormGetPrintMgrRefNum (formP);

   // Make sure refNum is not 0 (we should not be here if it is).
   ErrFatalDisplayIf ((refNum == 0), "Invalid state!");
   // Set orientation to landscape.
   PrnNewPage (refNum);
   // Return result.
   return err;
}

/*
   SampleSettings:
   Set default settings for the print job - orientation, font, etc.
   Parms:   refNum      Library reference number.
   Return:  0           Success
*/
Err
SampleSettings (FormPtr formP)
{
   // Initialize error.
   Err err = 0;
   
   // Retrieve PrintMgr reference number.
   UInt refNum = MainFormGetPrintMgrRefNum (formP);

   // Make sure refNum is not 0 (we should not be here if it is).
   ErrFatalDisplayIf ((refNum == 0), "Invalid state!");

   // Set orientation to landscape.
   PrnSetOrientation (refNum, prnOrientLandscape);
   
   // Return result.
   return err;
}

/*
   SampleVersion:
   Handle the main form's version command.
   Parms:   formP          Parent form.
   Return:  0              Success
*/
Err
SampleVersion (FormPtr formP)
{
   // Text buffers for messages.
   Char  msg [50];
   UInt  msgSize = sizeof (msg);
   
   // Retrieve reference number.
   UInt refNum = MainFormGetPrintMgrRefNum (formP);
   
   // Make sure refNum is not 0 (we should not be here if it is).
   ErrFatalDisplayIf ((refNum == 0), "Invalid state!");

   // Retrieve version from library.
   DWord dwVer;
   Err err = PrnGetLibAPIVersion (refNum, &dwVer);
   
   if (!err)
   {
   // Get API Version returned successfully.
   
      // Inform user of version.
      StrPrintF (msg, "Print Manager Version %lx.", dwVer);
      FrmCustomAlert (SuccessAlert, msg, NULL, NULL);
   }
   else
   {
   // Get API Version returned error.
   
      // Inform user of error.
      StrPrintF (msg, "Error %i retrieving Print Manager version.", err);
      FrmCustomAlert (ErrorAlert, msg, NULL, NULL);
   }
   
   // Return result.
   return err;
}
   
UInt
MainFormGetPrintMgrRefNum (FormPtr formP)
{
   // Get gadget index on form.
   Word gadgetIndex = FrmGetObjectIndex (formP, MainDataGadget);
   
   // Retrieve data associated with gadget.
   VoidPtr dataP = FrmGetGadgetData (formP, gadgetIndex);
   
   // Cast data pointer to UInt.
   UInt refNum = (UInt)dataP;
   
   return refNum;
}

void
MainFormSetPrintMgrRefNum (FormPtr formP, UInt refNum)
{
   // Get gadget index on form.
   Word gadgetIndex = FrmGetObjectIndex (formP, MainDataGadget);
   
   // Cast ref num to data pointer.
   VoidPtr dataP = (VoidPtr) refNum;
   
   // Set data associated with gadget.
   FrmSetGadgetData (formP, gadgetIndex, dataP);
}

void
MainFormEnableControls (FormPtr formP, PrintMgrStateEnum printMgrState)
{
   // Initialize enable control flags.
   Boolean enableLoad = false;
   Boolean enableOpen = false;
   Boolean enablePrint = false;
   Boolean enableClose = false;
   Boolean enableUnload = false;
   Boolean enableVersion = false;
   
   switch (printMgrState)
   {
      case printMgrLoaded :
      {
         enableOpen = true;
         enableVersion = true;
         enableUnload = true;
         break;
      }
      
      case printMgrOpened :
      {
         enablePrint = true;
         enableClose = true;
         enableVersion = true;
         break;
      }
      
      case printMgrUnloaded :
      {
         enableLoad = true;
         break;
      }
   }
   
   MainFormEnableControl (formP, MainLoadButton,     enableLoad);
   MainFormEnableControl (formP, MainPrintButton,    enablePrint);
   MainFormEnableControl (formP, MainCloseButton,    enableClose);
   MainFormEnableControl (formP, MainUnloadButton,   enableUnload);
   MainFormEnableControl (formP, MainOpenButton,   enableOpen);
   MainFormEnableControl (formP, MainVersionButton,  enableVersion);
}

void
MainFormEnableControl (FormPtr formP, Word objectID, Boolean enabled)
{
   // Get object index on form.
   Word objectIndex = FrmGetObjectIndex (formP, objectID);
   
   // Get pointer to control.
   ControlPtr controlP = (ControlPtr) FrmGetObjectPtr (formP, objectIndex);
   
   // Set enabled.
   CtlSetEnabled (controlP, enabled);
   
   if (enabled)
   {
      CtlShowControl (controlP);
   }
   else
   {
      CtlHideControl (controlP);
   }
}
